@@ -15,10 +15,11 @@ _MAX_BLOCK_SIZE = 20
 _MAX_INDENT_COUNT = 100
 ALLOWED_FUTURES = (
     'nested_scopes', 'generators', 'division', 'absolute_import',
-    'with_statement', 'print_function', 'unicode_literals',
+    'with_statement', 'print_function', 'unicode_literals', 'generator_stop',
 )
 _COMP_FOR_TYPES = ('comp_for', 'sync_comp_for')
 
+
 def _get_rhs_name(node, version):
     type_ = node.type
     if type_ == "lambdef":
@@ -39,7 +40,7 @@ def _get_rhs_name(node, version):
         elif (
             first == "("
             and (second == ")"
-            or (len(node.children) == 3 and node.children[1].type == "testlist_comp"))
+                 or (len(node.children) == 3 and node.children[1].type == "testlist_comp"))
         ):
             return "tuple"
         elif first == "(":
@@ -79,8 +80,7 @@ def _get_rhs_name(node, version):
             elif trailer.children[0] == ".":
                 return "attribute"
     elif (
-        ("expr" in type_
-        and "star_expr" not in type_)  # is a substring
+        ("expr" in type_ and "star_expr" not in type_)  # is a substring
         or "_test" in type_
         or type_ in ("term", "factor")
     ):
@@ -91,7 +91,8 @@ def _get_rhs_name(node, version):
         return "tuple"
     elif type_ == "fstring":
         return "f-string expression"
-    return type_ # shouldn't reach here
+    return type_  # shouldn't reach here
+
 
 def _iter_stmts(scope):
     """
@@ -146,8 +147,20 @@ def _remove_parens(atom):
     return atom
 
 
+def _skip_parens_bottom_up(node):
+    """
+    Returns an ancestor node of an expression, skipping all levels of parens
+    bottom-up.
+    """
+    while node.parent is not None:
+        node = node.parent
+        if node.type != 'atom' or node.children[0] != '(':
+            return node
+    return None
+
+
 def _iter_params(parent_node):
-    return (n for n in parent_node.children if n.type == 'param')
+    return (n for n in parent_node.children if n.type == 'param' or n.type == 'operator')
 
 
 def _is_future_import_first(import_from):
@@ -173,13 +186,11 @@ def _iter_definition_exprs_from_lists(exprlist):
             if child.children[0] == '(':
                 testlist_comp = child.children[1]
                 if testlist_comp.type == 'testlist_comp':
-                    for expr in _iter_definition_exprs_from_lists(testlist_comp):
-                        yield expr
+                    yield from _iter_definition_exprs_from_lists(testlist_comp)
                     return
                 else:
                     # It's a paren that doesn't do anything, like 1 + (1)
-                    for c in check_expr(testlist_comp):
-                        yield c
+                    yield from check_expr(testlist_comp)
                     return
             elif child.children[0] == '[':
                 yield testlist_comp
@@ -188,11 +199,9 @@ def _iter_definition_exprs_from_lists(exprlist):
 
     if exprlist.type in _STAR_EXPR_PARENTS:
         for child in exprlist.children[::2]:
-            for c in check_expr(child):  # Python 2 sucks
-                yield c
+            yield from check_expr(child)
     else:
-        for c in check_expr(exprlist):  # Python 2 sucks
-            yield c
+        yield from check_expr(exprlist)
 
 
 def _get_expr_stmt_definition_exprs(expr_stmt):
@@ -225,13 +234,14 @@ def _any_fstring_error(version, node):
         return search_ancestor(node, "fstring")
 
 
-class _Context(object):
+class _Context:
     def __init__(self, node, add_syntax_error, parent_context=None):
         self.node = node
         self.blocks = []
         self.parent_context = parent_context
         self._used_name_dict = {}
         self._global_names = []
+        self._local_params_names = []
         self._nonlocal_names = []
         self._nonlocal_names_in_subscopes = []
         self._add_syntax_error = add_syntax_error
@@ -255,6 +265,10 @@ class _Context(object):
             self._global_names.append(name)
         elif parent_type == 'nonlocal_stmt':
             self._nonlocal_names.append(name)
+        elif parent_type == 'funcdef':
+            self._local_params_names.extend(
+                [param.name.value for param in name.parent.get_params()]
+            )
         else:
             self._used_name_dict.setdefault(name.value, []).append(name)
 
@@ -282,6 +296,8 @@ class _Context(object):
         nonlocals_not_handled = []
         for nonlocal_name in self._nonlocal_names_in_subscopes:
             search = nonlocal_name.value
+            if search in self._local_params_names:
+                continue
             if search in global_name_strs or self.parent_context is None:
                 message = "no binding for nonlocal '%s' found" % nonlocal_name.value
                 self._add_syntax_error(nonlocal_name, message)
@@ -353,7 +369,7 @@ class ErrorFinder(Normalizer):
     Searches for errors in the syntax tree.
     """
     def __init__(self, *args, **kwargs):
-        super(ErrorFinder, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self._error_dict = {}
         self.version = self.grammar.version_info
 
@@ -377,7 +393,7 @@ class ErrorFinder(Normalizer):
                 # might find errors in there that should be ignored, because
                 # the error node itself already shows that there's an issue.
                 return ''
-        return super(ErrorFinder, self).visit(node)
+        return super().visit(node)
 
     @contextmanager
     def visit_node(self, node):
@@ -424,7 +440,9 @@ class ErrorFinder(Normalizer):
                         message = 'invalid syntax'
                         if (
                             self.version >= (3, 9)
-                            and leaf.value in _get_token_collection(self.version).always_break_tokens
+                            and leaf.value in _get_token_collection(
+                                self.version
+                            ).always_break_tokens
                         ):
                             message = "f-string: " + message
                     else:
@@ -440,7 +458,7 @@ class ErrorFinder(Normalizer):
                 self.context = self.context.add_context(parent)
 
         # The rest is rule based.
-        return super(ErrorFinder, self).visit_leaf(leaf)
+        return super().visit_leaf(leaf)
 
     def _add_indentation_error(self, spacing, message):
         self.add_issue(spacing, 903, "IndentationError: " + message)
@@ -466,7 +484,7 @@ class IndentationRule(Rule):
     code = 903
 
     def _get_message(self, message, node):
-        message = super(IndentationRule, self)._get_message(message, node)
+        message = super()._get_message(message, node)
         return "IndentationError: " + message
 
 
@@ -491,7 +509,7 @@ class SyntaxRule(Rule):
     code = 901
 
     def _get_message(self, message, node):
-        message = super(SyntaxRule, self)._get_message(message, node)
+        message = super()._get_message(message, node)
         if (
             "f-string" not in message
             and _any_fstring_error(self._normalizer.version, node)
@@ -589,9 +607,6 @@ class _NameChecks(SyntaxRule):
 
         if leaf.value == '__debug__' and leaf.is_definition():
             return True
-        if leaf.value == 'None' and self._normalizer.version < (3, 0) \
-                and leaf.is_definition():
-            self.add_issue(leaf, message=self.message_none)
 
 
 @ErrorFinder.register_rule(type='string')
@@ -601,7 +616,6 @@ class _StringChecks(SyntaxRule):
     def is_issue(self, leaf):
         string_prefix = leaf.string_prefix.lower()
         if 'b' in string_prefix \
-                and self._normalizer.version >= (3, 0) \
                 and any(c for c in leaf.value if ord(c) > 127):
             # b'ä'
             return True
@@ -609,14 +623,9 @@ class _StringChecks(SyntaxRule):
         if 'r' not in string_prefix:
             # Raw strings don't need to be checked if they have proper
             # escaping.
-            is_bytes = self._normalizer.version < (3, 0)
-            if 'b' in string_prefix:
-                is_bytes = True
-            if 'u' in string_prefix:
-                is_bytes = False
 
             payload = leaf._get_payload()
-            if is_bytes:
+            if 'b' in string_prefix:
                 payload = payload.encode('utf-8')
                 func = codecs.escape_decode
             else:
@@ -675,10 +684,6 @@ class _ReturnAndYieldChecks(SyntaxRule):
                 and any(self._normalizer.context.node.iter_yield_exprs()):
             if leaf.value == 'return' and leaf.parent.type == 'return_stmt':
                 return True
-            elif leaf.value == 'yield' \
-                    and leaf.get_next_leaf() != 'from' \
-                    and self._normalizer.version == (3, 5):
-                self.add_issue(self.get_node(leaf), message=self.message_async_yield)
 
 
 @ErrorFinder.register_rule(type='strings')
@@ -693,12 +698,10 @@ class _BytesAndStringMix(SyntaxRule):
 
     def is_issue(self, node):
         first = node.children[0]
-        # In Python 2 it's allowed to mix bytes and unicode.
-        if self._normalizer.version >= (3, 0):
-            first_is_bytes = self._is_bytes_literal(first)
-            for string in node.children[1:]:
-                if first_is_bytes != self._is_bytes_literal(string):
-                    return True
+        first_is_bytes = self._is_bytes_literal(first)
+        for string in node.children[1:]:
+            if first_is_bytes != self._is_bytes_literal(string):
+                return True
 
 
 @ErrorFinder.register_rule(type='import_as_names')
@@ -731,8 +734,6 @@ class _FutureImportRule(SyntaxRule):
             for from_name, future_name in node.get_paths():
                 name = future_name.value
                 allowed_futures = list(ALLOWED_FUTURES)
-                if self._normalizer.version >= (3, 5):
-                    allowed_futures.append('generator_stop')
                 if self._normalizer.version >= (3, 7):
                     allowed_futures.append('annotations')
                 if name == 'braces':
@@ -748,64 +749,70 @@ class _FutureImportRule(SyntaxRule):
 @ErrorFinder.register_rule(type='star_expr')
 class _StarExprRule(SyntaxRule):
     message_iterable_unpacking = "iterable unpacking cannot be used in comprehension"
-    message_assignment = "can use starred expression only as assignment target"
 
     def is_issue(self, node):
+        def check_delete_starred(node):
+            while node.parent is not None:
+                node = node.parent
+                if node.type == 'del_stmt':
+                    return True
+                if node.type not in (*_STAR_EXPR_PARENTS, 'atom'):
+                    return False
+            return False
+
+        if self._normalizer.version >= (3, 9):
+            ancestor = node.parent
+        else:
+            ancestor = _skip_parens_bottom_up(node)
+        # starred expression not in tuple/list/set
+        if ancestor.type not in (*_STAR_EXPR_PARENTS, 'dictorsetmaker') \
+                and not (ancestor.type == 'atom' and ancestor.children[0] != '('):
+            self.add_issue(node, message="can't use starred expression here")
+            return
+
+        if check_delete_starred(node):
+            if self._normalizer.version >= (3, 9):
+                self.add_issue(node, message="cannot delete starred")
+            else:
+                self.add_issue(node, message="can't use starred expression here")
+            return
+
         if node.parent.type == 'testlist_comp':
             # [*[] for a in [1]]
             if node.parent.children[1].type in _COMP_FOR_TYPES:
                 self.add_issue(node, message=self.message_iterable_unpacking)
-        if self._normalizer.version <= (3, 4):
-            n = search_ancestor(node, 'for_stmt', 'expr_stmt')
-            found_definition = False
-            if n is not None:
-                if n.type == 'expr_stmt':
-                    exprs = _get_expr_stmt_definition_exprs(n)
-                else:
-                    exprs = _get_for_stmt_definition_exprs(n)
-                if node in exprs:
-                    found_definition = True
-
-            if not found_definition:
-                self.add_issue(node, message=self.message_assignment)
 
 
 @ErrorFinder.register_rule(types=_STAR_EXPR_PARENTS)
 class _StarExprParentRule(SyntaxRule):
     def is_issue(self, node):
-        if node.parent.type == 'del_stmt':
-            if self._normalizer.version >= (3, 9):
-                self.add_issue(node.parent, message="cannot delete starred")
-            else:
-                self.add_issue(node.parent, message="can't use starred expression here")
-        else:
-            def is_definition(node, ancestor):
-                if ancestor is None:
-                    return False
+        def is_definition(node, ancestor):
+            if ancestor is None:
+                return False
 
-                type_ = ancestor.type
-                if type_ == 'trailer':
-                    return False
+            type_ = ancestor.type
+            if type_ == 'trailer':
+                return False
 
-                if type_ == 'expr_stmt':
-                    return node.start_pos < ancestor.children[-1].start_pos
+            if type_ == 'expr_stmt':
+                return node.start_pos < ancestor.children[-1].start_pos
 
-                return is_definition(node, ancestor.parent)
+            return is_definition(node, ancestor.parent)
 
-            if is_definition(node, node.parent):
-                args = [c for c in node.children if c != ',']
-                starred = [c for c in args if c.type == 'star_expr']
-                if len(starred) > 1:
-                    if self._normalizer.version < (3, 9):
-                        message = "two starred expressions in assignment"
-                    else:
-                        message = "multiple starred expressions in assignment"
-                    self.add_issue(starred[1], message=message)
-                elif starred:
-                    count = args.index(starred[0])
-                    if count >= 256:
-                        message = "too many expressions in star-unpacking assignment"
-                        self.add_issue(starred[0], message=message)
+        if is_definition(node, node.parent):
+            args = [c for c in node.children if c != ',']
+            starred = [c for c in args if c.type == 'star_expr']
+            if len(starred) > 1:
+                if self._normalizer.version < (3, 9):
+                    message = "two starred expressions in assignment"
+                else:
+                    message = "multiple starred expressions in assignment"
+                self.add_issue(starred[1], message=message)
+            elif starred:
+                count = args.index(starred[0])
+                if count >= 256:
+                    message = "too many expressions in star-unpacking assignment"
+                    self.add_issue(starred[0], message=message)
 
 
 @ErrorFinder.register_rule(type='annassign')
@@ -892,22 +899,10 @@ class _ArglistRule(SyntaxRule):
         arg_set = set()
         kw_only = False
         kw_unpacking_only = False
-        is_old_starred = False
-        # In python 3 this would be a bit easier (stars are part of
-        # argument), but we have to understand both.
         for argument in node.children:
             if argument == ',':
                 continue
 
-            if argument in ('*', '**'):
-                # Python < 3.5 has the order engraved in the grammar
-                # file.  No need to do anything here.
-                is_old_starred = True
-                continue
-            if is_old_starred:
-                is_old_starred = False
-                continue
-
             if argument.type == 'argument':
                 first = argument.children[0]
                 if _is_argument_comprehension(argument) and len(node.children) >= 2:
@@ -954,17 +949,28 @@ class _ParameterRule(SyntaxRule):
     def is_issue(self, node):
         param_names = set()
         default_only = False
+        star_seen = False
         for p in _iter_params(node):
+            if p.type == 'operator':
+                if p.value == '*':
+                    star_seen = True
+                    default_only = False
+                continue
+
             if p.name.value in param_names:
                 message = "duplicate argument '%s' in function definition"
                 self.add_issue(p.name, message=message % p.name.value)
             param_names.add(p.name.value)
 
-            if p.default is None and not p.star_count:
-                if default_only:
-                    return True
-            else:
-                default_only = True
+            if not star_seen:
+                if p.default is None and not p.star_count:
+                    if default_only:
+                        return True
+                elif p.star_count:
+                    star_seen = True
+                    default_only = False
+                else:
+                    default_only = True
 
 
 @ErrorFinder.register_rule(type='try_stmt')
@@ -1000,7 +1006,11 @@ class _FStringRule(SyntaxRule):
         if '\\' in expr.get_code():
             self.add_issue(expr, message=self.message_expr)
 
-        conversion = fstring_expr.children[2]
+        children_2 = fstring_expr.children[2]
+        if children_2.type == 'operator' and children_2.value == '=':
+            conversion = fstring_expr.children[3]
+        else:
+            conversion = children_2
         if conversion.type == 'fstring_conversion':
             name = conversion.children[1]
             if name.value not in ('s', 'r', 'a'):
@@ -1118,8 +1128,15 @@ class _CheckAssignmentRule(SyntaxRule):
                     error = "starred"
                 else:
                     self.add_issue(node, message="can't use starred expression here")
-            elif not search_ancestor(node, *_STAR_EXPR_PARENTS) and not is_aug_assign:
-                self.add_issue(node, message="starred assignment target must be in a list or tuple")
+            else:
+                if self._normalizer.version >= (3, 9):
+                    ancestor = node.parent
+                else:
+                    ancestor = _skip_parens_bottom_up(node)
+                if ancestor.type not in _STAR_EXPR_PARENTS and not is_aug_assign \
+                        and not (ancestor.type == 'atom' and ancestor.children[0] == '['):
+                    message = "starred assignment target must be in a list or tuple"
+                    self.add_issue(node, message=message)
 
             self._check_assignment(node.children[1])
 
@@ -1149,6 +1166,7 @@ class _CompForRule(_CheckAssignmentRule):
 class _ExprStmtRule(_CheckAssignmentRule):
     message = "illegal expression for augmented assignment"
     extended_message = "'{target}' is an " + message
+
     def is_issue(self, node):
         augassign = node.children[1]
         is_aug_assign = augassign != '=' and augassign.type != 'annassign'
@@ -1178,6 +1196,7 @@ class _ExprStmtRule(_CheckAssignmentRule):
                     ),
                 )
 
+
 @ErrorFinder.register_rule(type='with_item')
 class _WithItemRule(_CheckAssignmentRule):
     def is_issue(self, with_item):
